﻿using Hotline.Caching.Interfaces;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Snapshot;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Enums.User;
using Hotline.Snapshot;
using Hotline.Snapshot.Interfaces;
using Hotline.ThirdAccountDomainServices;
using Hotline.ThirdAccountDomainServices.Interfaces;
using IdentityModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Dependency;

namespace Hotline.Application.Snapshot;
public class SnapshotThirdAccountSupplier : IThirdAccountDomainService, IScopeDependency
{
    private readonly IThirdAccountRepository _thirdAccountRepository;
    private readonly IVolunteerRepository _volunteerRepository;
    private readonly ISnapshotUserInfoRepository _snapshotUserInfoRepository;
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;

    public SnapshotThirdAccountSupplier(IThirdAccountRepository thirdAccountRepository, IVolunteerRepository volunteerRepository, ISnapshotUserInfoRepository snapshotUserInfoRepository, ISystemSettingCacheManager systemSettingCacheManager)
    {
        _thirdAccountRepository = thirdAccountRepository;
        _volunteerRepository = volunteerRepository;
        _snapshotUserInfoRepository = snapshotUserInfoRepository;
        _systemSettingCacheManager = systemSettingCacheManager;
    }

    public async Task<List<Claim>> GetClaimAsync(ThirdAccount account, List<Claim> claims, CancellationToken token)
    {
        var userInfo = await _snapshotUserInfoRepository.GetByThirdIdAsync(account.Id);
        if(userInfo != null)
        {
            var subject = claims.Find(m => m.Type == JwtClaimTypes.Subject);
            if (subject != null)
                claims.Remove(subject);
            claims.Add(new(JwtClaimTypes.Subject, userInfo.Id));
        }
        return claims;
    }

    public async Task<Dictionary<string, object>> GetLoginOutDataAsync(ThirdAccount thirdAccount, Dictionary<string, object> dicOutData, CancellationToken cancel)
    {
        var isVolunteer = await _volunteerRepository.IsVolunteerAsync(thirdAccount.PhoneNumber);
        var user = await _snapshotUserInfoRepository.Queryable().Where(m => m.ThirdAccountId == thirdAccount.Id).FirstAsync();
        if (user == null)
        {
            user = new SnapshotUserInfo
            {
                CitizenType = EReadPackUserType.Citizen,
                ThirdAccountId = thirdAccount.Id,
                Name = thirdAccount.UserName,
                PhoneNumber = thirdAccount.PhoneNumber,
            };
            await _snapshotUserInfoRepository.AddAsync(user);
        }
        dicOutData.Add("UserType", user.CitizenType);
        dicOutData.Add("IsVolunteer", isVolunteer);
        dicOutData.Add("InvitationCode", user.InvitationCode);
        return dicOutData;
    }

    public async Task<ThirdTokenDto> GetThirdParameterAsync(ThirdTokenDto thirdDto, CancellationToken token)
    {
        if (thirdDto.ThirdType == EThirdType.WeChat)
        {
            thirdDto.AppId = _systemSettingCacheManager.WxOpenAppId;
            thirdDto.Secret = _systemSettingCacheManager.WxOpenAppSecret;
        }
        return await Task.FromResult(thirdDto);
    }

    public async Task RegisterAsync(ThirdAccount thirdAccount, CancellationToken token)
    {
        var userInfo = await _snapshotUserInfoRepository.GetByPhoneNumberAsync(thirdAccount.PhoneNumber);
        if (userInfo != null)
        {
            userInfo.CitizenType = EReadPackUserType.Guider;
            userInfo.ThirdAccountId = thirdAccount.Id;
            await _snapshotUserInfoRepository.UpdateAsync(userInfo);
        }
        else
        {
            userInfo = new SnapshotUserInfo
            {
                ThirdAccountId = thirdAccount.Id,
                PhoneNumber = thirdAccount.PhoneNumber,
                CitizenType = EReadPackUserType.Citizen,
            };
            await _snapshotUserInfoRepository.AddAsync(userInfo);
        }
    }
}

