﻿using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Novacode;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace Hotline.Share.Tools;
public static class DateTimeExtensions
{
    /// <summary>
    /// 获取两个时间之间的差
    /// </summary>
    /// <param name="beginDateTime">开始时间</param>
    /// <param name="endDateTime">结束时间</param>
    /// <returns>相距秒数</returns>
    public static int GetDifferenceSeconds(this DateTime beginDateTime, DateTime endDateTime)
    {
        return (int)(beginDateTime - endDateTime).TotalSeconds;
    }

    public static (DateTime, DateTime) DayStartAndEndTime(this DateTime dateTime)
    {
        var start = new DateTime(dateTime.Year, dateTime.Month, dateTime.Day);
        var end = start.AddDays(1).AddTicks(-1);
        return (start, end);
    }

    public static string ToLongDateTimeString(this DateTime dateTime)
    { 
        return dateTime.ToString("yyyy-MM-dd HH:mm:ss");
    }

    public static long ToUnixTimeMilliseconds(this DateTime value)
    {
        return new DateTimeOffset(value).ToUnixTimeMilliseconds();
    }

    public static EExpiredStatus? CalculateExpiredState(this DateTime? filedTime, EOrderStatus status, DateTime? expiredTime, DateTime? nearlyExpiredTime, DateTime? nearlyExpiredTimeOne)
    {
        DateTime? dateTime = DateTime.Now;
        if (status >= EOrderStatus.Filed)
        {
            dateTime = filedTime;
        }

        if (expiredTime.HasValue)
        {
            if (nearlyExpiredTimeOne.HasValue && dateTime < nearlyExpiredTimeOne)
            {
                return EExpiredStatus.Normal;
            }
            else if (!nearlyExpiredTimeOne.HasValue && dateTime < nearlyExpiredTime)
            {
                return EExpiredStatus.Normal;
            }
            else if (nearlyExpiredTimeOne.HasValue && dateTime > nearlyExpiredTimeOne && dateTime < nearlyExpiredTime)
            {
                return EExpiredStatus.GoingToExpiredOne;
            }
            else if (dateTime > nearlyExpiredTime && dateTime < expiredTime)
            {
                return EExpiredStatus.GoingToExpired;
            }
            else
            {
                return EExpiredStatus.Expired;
            }
        }

        return null;
    }

}
